function [S] = CalcTriStrains(sx, sy, sz, x, y, z, pr, ss, ts, ds)
% CalcTriStrains.m
%
% Calculates strains due to slip on a triangular dislocation in an
% elastic half space utilizing the symbolically differentiated
% displacement gradient tensor derived from the expressions for
% the displacements due to an angular dislocation in an elastic half
% space (Comninou and Dunders, 1975).
%
% Arguments
%  sx : x-coordinates of observation points
%  sy : y-coordinates of observation points
%  sz : z-coordinates of observation points
%  x  : x-coordinates of triangle vertices.
%  y  : y-coordinates of triangle vertices.
%  z  : z-coordinates of triangle vertices.
%  pr : Poisson's ratio
%  ss : strike slip displacement
%  ts : tensile slip displacement
%  ds : dip slip displacement
%
% Returns
%  S  : structure containing the strains (S.xx, S.yy, S.zz, S.xy, S.xz, S.yz)
%
% This paper should and related code should be cited as:
% Brendan J. Meade, Algorithms for the calculation of exact 
% displacements, strains, and stresses for Triangular Dislocation 
% Elements in a uniform elastic half space, Computers & 
% Geosciences (2007), doi:10.1016/j.cageo.2006.12.003.
%
% Use at your own risk and please let me know of any bugs/errors.
%
% Copyright (c) 2006 Brendan Meade
% 
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the
% "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.


% Calculate the slip vector in XYZ coordinates
normVec                      = cross([x(2);y(2);z(2)]-[x(1);y(1);z(1)], [x(3);y(3);z(3)]-[x(1);y(1);z(1)]);
normVec                      = normVec./norm(normVec);
if (normVec(3) < 0) % Enforce clockwise circulation
   normVec                   = -normVec;
   [x(2) x(3)]               = swap(x(2), x(3));
   [y(2) y(3)]               = swap(y(2), y(3));
   [z(2) z(3)]               = swap(z(2), z(3));
end
% %%-------------------------------------------------------
if (normVec(3) == 1)
    strikeVec = [1, 0, 0];
    dipVec    = [0, 1, 0];
    normVec   = [0, 0, 1];
else
    strikeVec                    = [-sin(atan2(normVec(2),normVec(1))) cos(atan2(normVec(2),normVec(1))) 0];
    dipVec                       = cross(normVec, strikeVec);
end

%%-------------------------------------------------------
% strikeVec                    = [-sin(atan2(normVec(2),normVec(1))) cos(atan2(normVec(2),normVec(1))) 0];
% dipVec                       = cross(normVec, strikeVec);
slipComp                     = [ss ds ts];
slipVec                      = [strikeVec(:) dipVec(:) normVec(:)] * slipComp(:);

% Solution vectors
S.xx                         = zeros(size(sx));
S.yy                         = zeros(size(sx));
S.zz                         = zeros(size(sx));
S.xy                         = zeros(size(sx));
S.xz                         = zeros(size(sx));
S.yz                         = zeros(size(sx));

% Add a copy of the first vertex to the vertex list for indexing
x(4)                         = x(1);
y(4)                         = y(1);
z(4)                         = z(1);

for iTri = 1:3
   % Calculate strike and dip of current leg
   strike                   = 180/pi*(atan2(y(iTri+1)-y(iTri), x(iTri+1)-x(iTri)));
   segMapLength             = sqrt((x(iTri)-x(iTri+1))^2 + (y(iTri)-y(iTri+1))^2);
   [rx ry]                  = RotateXyVec(x(iTri+1)-x(iTri), y(iTri+1)-y(iTri), -strike);
   dip                      = 180/pi*(atan2(z(iTri+1)-z(iTri), rx));
   
   if dip >= 0
      beta                  = pi/180*(90-dip);
      if beta > pi/2
         beta               = pi/2-beta;
      end
   else
      beta                  = -pi/180*(90+dip);
      if beta < -pi/2
         beta = pi/2-abs(beta);
      end
   end

   ssVec                    = [cos(strike/180*pi) sin(strike/180*pi) 0];
   tsVec                    = [-sin(strike/180*pi) cos(strike/180*pi) 0];
   dsVec                    = cross(ssVec, tsVec);
   lss                      = dot(slipVec, ssVec);
   lts                      = dot(slipVec, tsVec);
   lds                      = dot(slipVec, dsVec);

   if (abs(beta) > 0.000001) && (abs(beta-pi) > 0.000001)
      % First angular dislocation
      [sx1 sy1]                 = RotateXyVec(sx-x(iTri), sy-y(iTri), -strike);
      [a11 a22 a33 a12 a13 a23] = advs(sx1, sy1, sz-z(iTri), z(iTri), beta, pr, lss, lts, lds);
                                   
      % Second angular dislocation
      [sx2 sy2]                 = RotateXyVec(sx-x(iTri+1), sy-y(iTri+1), -strike); 
      [b11 b22 b33 b12 b13 b23] = advs(sx2, sy2, sz-z(iTri+1), z(iTri+1), beta, pr, lss, lts, lds);

      % Rotate tensors to correct for strike
      bxx                       = a11-b11;
      byy                       = a22-b22;
      bzz                       = a33-b33;
      bxy                       = a12-b12;
      bxz                       = a13-b13;
      byz                       = a23-b23;
      
      g                         = pi/180*strike;
      e11n                      = (cos(g)*bxx-sin(g)*bxy)*cos(g)-(cos(g)*bxy-sin(g)*byy)*sin(g);
      e12n                      = (cos(g)*bxx-sin(g)*bxy)*sin(g)+(cos(g)*bxy-sin(g)*byy)*cos(g);
      e13n                      = cos(g)*bxz-sin(g)*byz;
      e22n                      = (sin(g)*bxx+cos(g)*bxy)*sin(g)+(sin(g)*bxy+cos(g)*byy)*cos(g);
      e23n                      = sin(g)*bxz+cos(g)*byz;
      e33n                      = bzz;
      
      % Add the strains from current leg
      S.xx                      = S.xx + e11n;
      S.yy                      = S.yy + e22n;
      S.zz                      = S.zz + e33n;
      S.xy                      = S.xy + e12n;
      S.xz                      = S.xz + e13n;
      S.yz                      = S.yz + e23n;
   end
end


function [a b] = swap(a, b)
% Swap two values
temp                            = a;
a                               = b;
b                               = temp;


function [xp yp] = RotateXyVec(x, y, alpha)
% Rotate a vector by an angle alpha
x                             = x(:);
y                             = y(:);
alpha                         = pi/180*alpha;
xp                            = cos(alpha).*x - sin(alpha).*y;
yp                            = sin(alpha).*x + cos(alpha).*y;


function [e11 e22 e33 e12 e13 e23] = advs(y1, y2, y3, a, b, nu, B1, B2, B3)
% These are the strains in a uniform elastic half space due to slip
% on an angular dislocation.  They were calculated by symbolically
% differentiating the expressions for the displacements (Comninou and
% Dunders, 1975, with typos noted by Thomas 1993) then combining the
% elements of the displacement gradient tensor to form the strain tensor.

e11 = B1.*(1./8.*((2-2.*nu).*(2.*y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)-y3.*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)-y3.*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))-y2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y1-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y1)-y2.*cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))))./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*(y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2-(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1.*cot(b)-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))-(1-2.*nu).*y2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(1-2.*nu).*y2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-3.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+a./(y1.^2+y2.^2+(y3+2.*a).^2)-2.*a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1.*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1.*cot(b)+(2-2.*nu).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1).*cos(b))+2.*a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1))./pi./(1-nu))+B2.*(1./8.*((-1+2.*nu).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))+2.*y1.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+y1.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y1-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y1)+cos(b).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(y1.*cos(b)-y3.*sin(b)).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))+cos(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+(y1.*cos(b)+(y3+2.*a).*sin(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2+nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1).*cos(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*y1.*cot(b)+nu.*(y3+2.*a)-a+a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+a.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-a.*y1.^2.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+2.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y1.^3./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.^3./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))+(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((y1.*cos(b)+(y3+2.*a).*sin(b)).*cos(b)-a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b).^2-a.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)+a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y1)-a.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+3.*a.*y1.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)+a)-y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)+a).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1-2.*nu).*cot(b)-2.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y1.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y1.^3./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y1.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1+3.*a.*y1.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2))-(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(-cos(b).*sin(b)+a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)))).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)-3.*a.*y1.^2.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)./cos(b)+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))).*y1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y1)))./pi./(1-nu))+B3.*(1./8.*y2.*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y1-1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))./pi./(1-nu)+1./4.*((1-2.*nu).*(-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1)-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1))./pi./(1-nu));
e22 = B1.*(1./8.*((1-2.*nu).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))-2.*y2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))-y2.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y2-cos(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2)))./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2-nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1-2.*nu).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(y1.*cot(b).*(1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+nu.*(y3+2.*a)-a+y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2+2.*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^3./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y2.^3./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))+(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2+3.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(-2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)-a)+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)-a).*y2+2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^3./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a+2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-3.*a.*y2.^3./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).^2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b))+a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b)).*y2-3.*a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y2.*cos(b).^2+a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*y2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2)))./pi./(1-nu))+B2.*(1./8.*((2-2.*nu).*(-2./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)-y3.*sin(b))./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+y1.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y2)-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-y2.*(-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2))./pi./(1-nu)+1./4.*((2-2.*nu).*(1-2.*nu).*(-1./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))-(1-2.*nu).*y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2.*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)-(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+(1-2.*nu).*y2.^2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+(1-2.*nu).*y2.^2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)-a.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+3.*a.*y2.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*y2-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2))+(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b))-y2.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b))-y2.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b))+y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y2-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2./cos(b).*y2))./pi./(1-nu))+B3.*(1./8.*((1-2.*nu).*sin(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-2.*y2.*sin(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y2.^2.*sin(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2))./pi./(1-nu)+1./4.*((1-2.*nu).*(-sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*y1+y1.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(sin(b).*(cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(sin(b).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y2-2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*a.*(y3+2.*a).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y2.*cos(b).*sin(b)+a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*y2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2)))./pi./(1-nu));
e33 = B1.*(1./8.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y3+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*cos(b).*y3-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*((1-2.*nu).*(-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b)-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))+y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a))+y2.*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-a./(y1.^2+y2.^2+(y3+2.*a).^2)+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)))./pi./(1-nu))+B2.*(1./8.*((-1+2.*nu).*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y1.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y3+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))-sin(b).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(y1.*cos(b)-y3.*sin(b)).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*cos(b).*y3-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))-sin(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*cot(b).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+(2-2.*nu).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+1./2.*(2-2.*nu).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+(2-2.*nu).*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a))-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b).*sin(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).*sin(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*(2.*y3+4.*a)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))-1./2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*(2.*y3+4.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y3+2.*a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2)+(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)-sin(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))))./pi./(1-nu))+B3.*(1./8.*((2-2.*nu).*(y2./(y1.*cos(b)-y3.*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y3+y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)+y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)-(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+y2.*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*cos(b).*y3-1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*(-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))-(2-2.*nu).*y2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*(2-2.*nu).*y2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+y2.*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)-y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2)-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)))./pi./(1-nu));
e12 = 1./2.*B1.*(1./8.*((2-2.*nu).*(-2./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)-y3.*sin(b))./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))-y1.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y2)-cos(b).*(((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y2.*cos(b).*(1./(y1.^2+y2.^2+y3.^2).*sin(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2))./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*(-1./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))-(1-2.*nu).*y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2.*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)+(1-2.*nu).*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(1-2.*nu).*y2.^2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-(1-2.*nu).*y2.^2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+a.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-3.*a.*y2.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-2.*a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2)+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2.*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2.*cot(b)+(2-2.*nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*y2.*cos(b))+2.*a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2))./pi./(1-nu))+1./2.*B2.*(1./8.*((-1+2.*nu).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))+y1.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y2-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y2)+(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).*sin(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2)./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2+nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*y1.*cot(b)+nu.*(y3+2.*a)-a+a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2)+(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((y1.*cos(b)+(y3+2.*a).*sin(b)).*cos(b)-a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-a./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*y2./cos(b)+a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y2)+3.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)+a)-y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)+a).*y2+y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*y2+y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*y2+y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a.*y2+3.*a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y2)-(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(-cos(b).*sin(b)+a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-3.*a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)./cos(b).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*y2.*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y2)))./pi./(1-nu))+1./2.*B3.*(1./8.*sin(b).*(((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))./pi./(1-nu)+1./8.*y2.*sin(b).*(1./(y1.^2+y2.^2+y3.^2).*sin(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2)./pi./(1-nu)+1./4.*((1-2.*nu).*(1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y2.^2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y2.^2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))-(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2)+(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2))./pi./(1-nu))+1./2.*B1.*(1./8.*((1-2.*nu).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))-y2.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y1-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y1-cos(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))))./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2-nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1-2.*nu).*cos(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(y1.*cot(b).*(1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+nu.*(y3+2.*a)-a+y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+a.*y1.^2.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)-(1-2.*nu).*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-a.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+3.*a.*y1.^2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(-2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)-a)+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)-a).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1-2.*nu).*cot(b)-y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*y1-y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*y1-y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a.*y1-3.*a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1)-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).^2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b))+a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a))).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b)).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1-2.*nu).*cos(b).*cot(b)+a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-3.*a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-a.*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)+a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*y1-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1)))./pi./(1-nu))+1./2.*B2.*(1./8.*((2-2.*nu).*(2.*y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)-y3.*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)-y3.*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+y2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y1-1./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*y1)-y2.*(cos(b)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))+cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*(1-2.*nu).*(y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2-(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1.*cot(b)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))+(1-2.*nu).*y2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+(1-2.*nu).*y2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y1+3.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))).*y1-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))).*y1+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+2.*nu.*y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1))-y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b)).*y1-y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y1-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2./cos(b).*y1))./pi./(1-nu))+1./2.*B3.*(1./8.*((1-2.*nu).*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y2.^2.*sin(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))))./pi./(1-nu)+1./4.*((1-2.*nu).*(-sin(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-y1.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+y1.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(sin(b).*(cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a))).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(sin(b).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1+cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*a.*(y3+2.*a).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-a.*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)+a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*y1-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1)))./pi./(1-nu));
e13 = 1./2.*B1.*(1./8.*((2-2.*nu).*(y2./(y1.*cos(b)-y3.*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y3+y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))-y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1))-y2.*cos(b).*(1./(y1.^2+y2.^2+y3.^2).*sin(b).*y3./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*(2.*y3+4.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))))./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*(-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2-(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(1./2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+1./2.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))-(1-2.*nu).*y2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*(1-2.*nu).*y2.*cos(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2.*cot(b)-3./2.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a)+y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a))+y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b))-a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+(2-2.*nu).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1).*cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1).*((1-2.*nu).*cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a).*cot(b)+1./2.*(2-2.*nu)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).*(2.*y3+4.*a).*cos(b))-a.*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2)+a.*(y3+2.*a).*cos(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)))./pi./(1-nu))+1./2.*B2.*(1./8.*((-1+2.*nu).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))+y1.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1))-sin(b).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).*sin(b).*y3./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))+sin(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*(2.*y3+4.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2+nu).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1).*cos(b).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu).*y1.*cot(b)+nu.*(y3+2.*a)-a+a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu-1./2.*a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))+(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((y1.*cos(b)+(y3+2.*a).*sin(b)).*cos(b)-a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-(1-2.*nu).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b).*sin(b)-1./2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*(2.*y3+4.*a)./cos(b)+1./2.*a.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*(2.*y3+4.*a))-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1.*cot(b)+3./2.*a.*y1.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)+a)-y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)+a)-y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)+a).*(2.*y3+4.*a)+1./2.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(2.*y3+4.*a)+y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+1./2.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a.*(2.*y3+4.*a)+3./2.*a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a))+cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-cos(b).*sin(b)+a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))))-(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(-cos(b).*sin(b)+a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+(y3+a).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*y1-3./2.*a.*y1.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2)./cos(b).*(2.*y3+4.*a)+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*(2.*y3+4.*a).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))).*(2.*y3+4.*a)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*(2.*y3+4.*a))))./pi./(1-nu))+1./2.*B3.*(1./8.*y2.*sin(b).*(1./(y1.^2+y2.^2+y3.^2).*sin(b).*y3./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).*(2.*y3+4.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)-y1)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))./pi./(1-nu)+1./4.*((1-2.*nu).*(-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))-y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))+1./2.*y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*(2.*y3+4.*a)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1))+y2.*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2)-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)))./pi./(1-nu))+1./2.*B1.*(1./8.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-cos(b).*(1./(y1.^2+y2.^2+y3.^2).*cos(b).*y1./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*((1-2.*nu).*(y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1)-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1+2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1))./pi./(1-nu))+1./2.*B2.*(1./8.*((-1+2.*nu).*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-1./(y1.^2+y2.^2+y3.^2).^(1./2)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y1+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1)+cos(b).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).*cos(b).*y1./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))-cos(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*cot(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-(2-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(2-2.*nu).*y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+(2-2.*nu).*y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+(2-2.*nu).*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+2.*nu.*y1.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-a./(y1.^2+y2.^2+(y3+2.*a).^2)+2.*a.*y1.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).*sin(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1.*cot(b).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*y1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*y1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-(y3+2.*a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2)+2.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1-cos(b).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b)))))./pi./(1-nu))+1./2.*B3.*(1./8.*((2-2.*nu).*(-y2./(y1.*cos(b)-y3.*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)-y3.*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)+y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)-(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+y2.*sin(b).*(1./(y1.^2+y2.^2+y3.^2).*cos(b).*y1./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y1-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y1-sin(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*(y2./y1.^2./(1+y2.^2./y1.^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*cos(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*y1-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*(2.*y1.*cos(b)+(y3+2.*a).*sin(b)))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))-(2-2.*nu).*y2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-(2-2.*nu).*y2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*y1-y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))+y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y1-sin(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y1))./pi./(1-nu));
e23 = 1./2.*B1.*(1./8.*((1-2.*nu).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3)+(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))-y2.^2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-cos(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b)))))./pi./(1-nu)+1./4.*((1-2.*nu).*(((2-2.*nu).*cot(b).^2-nu).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-((2-2.*nu).*cot(b).^2+1-2.*nu).*cos(b).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(y1.*cot(b).*(1-2.*nu-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+nu.*(y3+2.*a)-a+y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-(1-2.*nu)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(1./2.*a.*y1.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+nu-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))-(1-2.*nu).*sin(b).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*(1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y1.*cot(b)+3./2.*a.*y1.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)-a)+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(-2.*nu+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*y1.*cot(b)-a)+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*y1.*cot(b)-a).*(2.*y3+4.*a)-1./2.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(2.*y3+4.*a)-y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a.*(2.*y3+4.*a)-3./2.*a.*y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a))+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b).^2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b))+a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).^2-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b))+a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)+a.*cos(b)).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1-2.*nu).*sin(b).*cot(b)+a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+a.*(y3+2.*a).*sin(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-3./2.*a.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a)+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(2.*y3+4.*a)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).^2-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-a.*sin(b).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)+1./2.*a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*(2.*y3+4.*a)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b)).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1))))./pi./(1-nu))+1./2.*B2.*(1./8.*((2-2.*nu).*(y2./(y1.*cos(b)-y3.*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+(y2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).*y3+y2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+y1.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y3).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-1)-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1))-y2.*(-sin(b)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))+sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))-1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))))./pi./(1-nu)+1./4.*((2-2.*nu).*(1-2.*nu).*(-y2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2.*sin(b)./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+(1./2.*y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).*(2.*y3+4.*a)-y2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*y1)./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b).^2-(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((-1+2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*cot(b)+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+(1-2.*nu).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(-1./2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a).*cot(b)-y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)-1./2.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))+(1-2.*nu).*y2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+1./2.*(1-2.*nu).*y2.*cot(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*(2.*y3+4.*a)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2.*cot(b)+3./2.*a.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(5./2).*(2.*y3+4.*a)+y2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)))-1./2.*y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))).*(2.*y3+4.*a)-y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+1./2.*a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*(2.*y3+4.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)))+y2.*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b))-1./2.*y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b)).*(2.*y3+4.*a)-y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*((-2+2.*nu).*cos(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))+y2.*(y3+a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*((1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./cos(b)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./cos(b).*(2.*y3+4.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2)./cos(b)-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2./cos(b).*(2.*y3+4.*a)))./pi./(1-nu))+1./2.*B3.*(1./8.*((1-2.*nu).*sin(b).*((1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))+(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y2.^2.*sin(b).*(-1./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y3-1./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y3-cos(b))-1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(2.*y3+4.*a)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))))./pi./(1-nu)+1./4.*((1-2.*nu).*(-sin(b).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1)+1./2.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a))+y1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a))-1./2.*y1.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)).*(2.*y3+4.*a)+y1.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a)-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+1))-1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(sin(b).*(cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)))+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(sin(b).*(cos(b)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a))).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./2.*sin(b).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*y3+4.*a)+sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-1./2.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(1+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)).*(2.*y3+4.*a)+(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(a./(y1.^2+y2.^2+(y3+2.*a).^2)-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*(2.*y3+4.*a))+1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(2.*y3+4.*a)+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(y2.^2.*cos(b).*sin(b)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*y3+4.*a)+cos(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-a.*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)+1./2.*a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*(2.*y3+4.*a)-a.*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(1./2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*(2.*y3+4.*a)+1))))./pi./(1-nu))+1./2.*B1.*(1./8.*(1./(y1.^2+y2.^2+y3.^2).^(1./2)-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-cos(b).*(((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))./pi./(1-nu)+1./8.*y2.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-cos(b).*(1./(y1.^2+y2.^2+y3.^2).*cos(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2))./pi./(1-nu)+1./4.*((2-2.*nu).*((1-2.*nu).*(-1./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2)).*cot(b)+1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y2.^2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)-cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+y2.^2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y2.^2.*cos(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2))+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)+a./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-2.*nu./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-2.*a./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2)+(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1-2.*nu-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a).*cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2+2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2))./pi./(1-nu))+1./2.*B2.*(1./8.*((-1+2.*nu).*sin(b).*(1./(y1.^2+y2.^2+y3.^2).^(1./2).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-y1.*(-1./(y1.^2+y2.^2+y3.^2).^(3./2).*y2+1./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2)+(y1.*cos(b)-y3.*sin(b))./(y1.^2+y2.^2+y3.^2).*cos(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-(y1.*cos(b)-y3.*sin(b)).*((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2)./pi./(1-nu)+1./4.*((-2+2.*nu).*(1-2.*nu).*cot(b).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+(2-2.*nu).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2.*(2.*nu+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+(2-2.*nu).*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(2-2.*nu).*(y1.*cos(b)+(y3+2.*a).*sin(b))./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((1-2.*nu).*cot(b)-2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a)-a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2)).*y2+(y3+a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*nu.*y1./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)+y3+2.*a).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2+2.*a.*y1./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2)+(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b).*sin(b)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-(y3+a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2.*cot(b).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*((2-2.*nu).*cos(b)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a).*cot(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(-cos(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2)-a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*(sin(b)-(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2)-(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))).*y2+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*(2.*(y3+2.*a).*(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2-(y1.*cos(b)+(y3+2.*a).*sin(b))./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2+(y1.*cos(b)+(y3+2.*a).*sin(b)).*((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2)))./pi./(1-nu))+1./2.*B3.*(1./8.*((2-2.*nu).*(1./(y1.*cos(b)-y3.*sin(b))./(1+y2.^2./(y1.*cos(b)-y3.*sin(b)).^2)+((y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+y3.^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+y3.^2).*sin(b).^2./(y1.*(y1.*cos(b)-y3.*sin(b))+y2.^2.*cos(b)).^2)-1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+sin(b).*(((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(1./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)))+y2.*sin(b).*(1./(y1.^2+y2.^2+y3.^2).*cos(b).*y2./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b))-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2).^(3./2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).*y2-((y1.^2+y2.^2+y3.^2).^(1./2).*cos(b)-y3)./(y1.^2+y2.^2+y3.^2)./((y1.^2+y2.^2+y3.^2).^(1./2)-y1.*sin(b)-y3.*cos(b)).^2.*y2-1./(y1.^2+y2.^2+(y3+2.*a).^2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b))+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*y2+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*y2))./pi./(1-nu)+1./4.*((2-2.*nu).*(-1./y1./(1+y2.^2./y1.^2)+1./(y1.*cos(b)+(y3+2.*a).*sin(b))./(1+y2.^2./(y1.*cos(b)+(y3+2.*a).*sin(b)).^2)+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))+y2.^2./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b))-2.*y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*sin(b)./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2.*cos(b))./(1+y2.^2.*(y1.^2+y2.^2+(y3+2.*a).^2).*sin(b).^2./(y1.*(y1.*cos(b)+(y3+2.*a).*sin(b))+y2.^2.*cos(b)).^2))+(2-2.*nu).*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-(2-2.*nu).*y2.^2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-(2-2.*nu).*y2.^2.*sin(b)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)+(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))-y2.^2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(1+((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))+a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2))+y2.*(y3+a).*sin(b)./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(1./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b).*y2./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).^2.*(cos(b)+a./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2))./(y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*y2-((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2).*cos(b)+y3+2.*a)./((y1.^2+y2.^2+(y3+2.*a).^2).^(1./2)-y1.*sin(b)+(y3+2.*a).*cos(b)).*a./(y1.^2+y2.^2+(y3+2.*a).^2).^(3./2).*y2-2.*a.*(y3+2.*a)./(y1.^2+y2.^2+(y3+2.*a).^2).^2.*y2))./pi./(1-nu));


